/* Our mouse repeater object */
objectdef mouserepeater
{
	/* This is our constructor. It makes sure the events we're attaching to are registered, so that we can attach to them later. */
	method Initialize()
	{
		LavishScript:RegisterEvent[OnButtonMove]
		LavishScript:RegisterEvent[OnMouseMove]
		LavishScript:RegisterEvent[OnMouseWheel]
	}
 
	/* This variable stores whether it is currently enabled or not, so we can have our handy Toggle method. */
	variable bool Enabled=FALSE
 
	/* Mouse1Down and Mouse2Down keep the button state of Mouse1 and Mouse2 (left and right) 
           so we know whether to stop sending relative mouse movements. We only want them if a button is down (i.e. for mouselook)
	*/
	variable bool Mouse1Down=FALSE
	variable bool Mouse2Down=FALSE
 
	/* This is an event handler for when a button is pressed or released. That includes keyboard and mouse, as well as alternative input devices like a Saitek pad or G15 keys. 
	   The event handler will only be called when our state is Enabled. */
	method ButtonMoved(string Button, bool Position)
	{
		/* This echo is commented out, but you can un-comment it to see messages when a button is pressed or released */
;		echo Button=${Button} position=${Position}
 
		/* Use a switch to efficiently handle special cases */
		switch ${Button}
		{
		/* For mouse buttons 1 and 2, we also want the mouse position if the cursor is visible */
		case Mouse1
			if ${Position}
			{
				/* Attach to the OnMouseMove event, because until we release the button, we want to relay relative mouse movements */
				Mouse1Down:Set[TRUE]
 
				/* And only send the cursor position if the cursor is currently visible... */
				if ${Mouse.Cursor}
					relay "all other" "mouseto ${Mouse};mouseclick -hold left"
				else
				{
					relay "all other" mouseclick -hold left
				}
 
			}
			else
			{
				/* If neither button is held down now, stop relaying relative mouse movements */
				Mouse1Down:Set[FALSE]
 
				/* And only send the cursor position if the cursor is currently visible... */
				if ${Mouse.Cursor}
					relay "all other" "mouseto ${Mouse};mouseclick -release left"
				else
					relay "all other" mouseclick -release left
			}
			break
		case Mouse2
			if ${Position}
			{
				/* Attach to the OnMouseMove event, because until we release the button, we want to relay relative mouse movements */
				Mouse2Down:Set[TRUE]
 
				/* And only send the cursor position if the cursor is currently visible... */
				if ${Mouse.Cursor}
					relay "all other" "mouseto ${Mouse};mouseclick -hold right"
				else
				{
					relay "all other" mouseclick -hold right
				}
			}
			else
			{
				/* If neither button is held down now, stop relaying relative mouse movements */
				Mouse2Down:Set[FALSE]
 
				/* And only send the cursor position if the cursor is currently visible... */
				if ${Mouse.Cursor}
					relay "all other" "mouseto ${Mouse};mouseclick -release right"
				else
					relay "all other" mouseclick -release right
			}
			break
		case Mouse3		case Mouse4
		case Mouse5
		case Mouse6
		case Mouse7
		case Mouse8
			/* In the default case for other mouse buttons, we want "all other" windows to repeat the same action. */
			if ${Position}
				relay "all other" press -hold "${Button.Escape}"
			else
				relay "all other" press -release "${Button.Escape}"
			break
		}
	}
 
	/* While left or right mouse button is held down, this will send relative mouse position changes */
	method OnMouseMove(int dX, int dY)
	{
		if ${Mouse1Down} || ${Mouse2Down}
			relay "all other" mouseto -relative ${dX},${dY}
		else
			relay "all other" mouseto ${Mouse}
	}

	/* While mouse repeating is enabled, this will send mousewheel movement */
	method OnMouseWheel(int offset)
	{
		relay "all other" mousewheel ${offset}
	}
 
 
	/* This is the easiest thing to bind to. One press to turn it on, one press to turn it off, no extra work. */
	method Toggle()
	{
		if ${Enabled}
			This:Disable
		else
			This:Enable
	}
 
	/* This will flat out Enable the repeater. If it's already enabled, nothing will change. */
	method Enable()
	{
		Event[OnButtonMove]:AttachAtom[This:ButtonMoved]
		Event[OnMouseWheel]:AttachAtom[This:OnMouseWheel]
		Event[OnMouseMove]:AttachAtom[This:OnMouseMove]
		UIElement[mousetoggle@main@repeater]:SetText["*Mouse*"]
		Enabled:Set[TRUE]	
	}
 
	/* This will flat out Disable the repeater. If it's already disabled, nothing will change. */
	method Disable()
	{
		Event[OnButtonMove]:DetachAtom[This:ButtonMoved]		
		Event[OnMouseWheel]:DetachAtom[This:OnMouseWheel]
		Event[OnMouseMove]:DetachAtom[This:OnMouseMove]
		UIElement[mousetoggle@main@repeater]:SetText["Mouse"]
		Enabled:Set[FALSE]
	}
 
}
 
/* Our key repeater object */
objectdef keyrepeater
{
	/* This is our constructor. It makes sure the events we're attaching to are registered, so that we can attach to them later. */
	method Initialize()
	{
		LavishScript:RegisterEvent[OnButtonMove]
		LavishSettings:AddSet["Repeater Keylists"]
		ListRoot:Set[${LavishSettings.FindSet["Repeater Keylists"]}]
	}
 
	method Shutdown()
	{
		ListRoot:Remove
	}
 
	/* This variable stores whether it is currently enabled or not, so we can have our handy Toggle method. */
	variable bool Enabled=FALSE
 
	variable settingsetref ListRoot
	variable settingsetref KeyList
	variable bool WhiteList
 
	/* This is an event handler for when a button is pressed or released. That includes keyboard and mouse, as well as alternative input devices like a Saitek pad or G15 keys. 
	   The event handler will only be called when our state is Enabled. */
	method ButtonMoved(string Button, bool Position)
	{
		/* This echo is commented out, but you can un-comment it to see messages when a button is pressed or released */
;		echo Button=${Button} position=${Position}
 
		/* Use a switch to efficiently handle special cases */
		switch ${Button}
		{
		/* Ignore mouse buttons, those are for the mouse repeater to handle */
		case Mouse1
		case Mouse2
		case Mouse3
		case Mouse4
		case Mouse5
		case Mouse6
		case Mouse7
		case Mouse8
			break
		default
			/* Determine if we are using a keylist */
			if ${KeyList.Name(exists)}
			{
				/* is it a white or black list? */
				if ${WhiteList}
				{
					/* whitelist. ignore key if it is NOT in the list */
					if !${KeyList.FindSetting["${Button.Escape}"](exists)}
						break
				}
				else
				{
					/* blacklist. ignore key if it is in the list. */
					if ${KeyList.FindSetting["${Button.Escape}"](exists)}
						break
				}
			}
 
			/* In the default case, we want "all other" windows to repeat the same action. */
			if ${Position}
				relay "all other" press -hold "${Button.Escape}"
			else
				relay "all other" press -release "${Button.Escape}"
			break
		}
	}
 
	method RemoveList(string name)
	{
		ListRoot.FindSet["${name.Escape}"]:Remove
	}
 
	method ImportLists(string Filename)
	{
		ListRoot:Import["${Filename}"]
	}
 
	method ExportLists(string Filename)
	{
		ListRoot:Export["${Filename}"]
	}
 
	method SetList(bool White, string Name)
	{
		WhiteList:Set[${White}]
		KeyList:Set[${ListRoot.FindSet["${Name.Escape}"]}]
	}
 
	method UnsetList()
	{
		KeyList:Set[0]
	}
 
	/* This is the easiest thing to bind to. One press to turn it on, one press to turn it off, no extra work. */
	method Toggle()
	{
		if ${Enabled}
			This:Disable
		else
			This:Enable
	}
 
	/* This will flat out Enable the repeater. If it's already enabled, nothing will change. */
	method Enable()
	{
		Event[OnButtonMove]:AttachAtom[This:ButtonMoved]
		UIElement[keytoggle@main@repeater]:SetText["*Keys*"]
		Enabled:Set[TRUE]	
	}
 
	/* This will flat out Disable the repeater. If it's already disabled, nothing will change. */
	method Disable()
	{
		Event[OnButtonMove]:DetachAtom[This:ButtonMoved]		
		UIElement[keytoggle@main@repeater]:SetText["Keys"]
		Enabled:Set[FALSE]
	}
 
}
 
/* Our instance of the keyrepeater class. Our object is called KeyRepeater */
variable keyrepeater KeyRepeater
/* Our instance of the mouserepeater class. Our object is called MouseRepeater */
variable mouserepeater MouseRepeater
 
/* This creates a command that will until the repeater script is terminated, called repeater. It can turn on, toggle, or turn off the key or mouse repeaters.
   Examples:
	repeater on
	repeater off
	repeater toggle
	repeater keyon
	repeater keyoff
	repeater keytoggle
	repeater mouseon
	repeater mouseoff
	repeater mousetoggle
 */
atom(global) repeater(... Params)
{
	variable int Count=1
	for (${Count}<=${Params.Size} ; Count:Inc)
	{
		/* Check the variable with a switch.. */
		switch ${Params[${Count}]}
		{
		case import
			Count:Inc
			if ${Count}<=${Params.Size}
			{
				KeyRepeater:ImportLists["${Params[${Count}].Escape}"]
				echo Repeater key lists imported from ${Params[${Count}]}
			}
			break
		case export
			Count:Inc
			if ${Count}<=${Params.Size}
			{
				KeyRepeater:ExportLists["${Params[${Count}].Escape}"]
				echo Repeater key lists exported to ${Params[${Count}]}
			}
			break
		case whitelist
			Count:Inc
			if ${Count}<=${Params.Size}
			{
				KeyRepeater:SetList[TRUE,"${Params[${Count}].Escape}"]
				echo Repeater whitelist: ${Params[${Count}]}
			}
			break
		case blacklist
			Count:Inc
			if ${Count}<=${Params.Size}
			{
				KeyRepeater:SetList[FALSE,"${Params[${Count}].Escape}"]
				echo Repeater blacklist: ${Params[${Count}]}
			}
			break
		case removelist
			Count:Inc
			if ${Count}<=${Params.Size}
			{
				echo Removing Repeater key list ${Params[${Count}]}
				KeyRepeater:RemoveList["${Params[${Count}].Escape}"]
			}
			break
		case nolist
			KeyRepeater:UnsetList
			echo Repeater white/black list cleared
			break
		case on
			KeyRepeater:Enable
			MouseRepeater:Enable
			break
		case toggle
			KeyRepeater:Toggle
			MouseRepeater:Toggle
			break
		case off
			KeyRepeater:Disable
			MouseRepeater:Disable
			break
		case mouseon
			MouseRepeater:Enable
			break
		case mouseoff
			MouseRepeater:Disable
			break
		case mousetoggle
			MouseRepeater:Toggle
			break
		case keyon
			KeyRepeater:Enable
			break
		case keyoff
			KeyRepeater:Disable
			break
		case keytoggle
			KeyRepeater:Toggle
			break
		}
	}
}
 
/* The entry point for the script. Any parameters will be turned into a string array called Params */
function main(... Params)
{
	if ${Params.Size}
	{
		/* Parameters were passed, so let's assume that the first one is the key we wanted to bind to. */
		bind repeater "${Params[1]}" repeater toggle
	}
	else
	{
		/* Bind to MR by default (One of the G15/G11 keys) */
		bind repeater MR repeater toggle
	}
 
	repeater import "repeater lists.xml"
 
	/* Now just keep our script running. */
	while 1
		waitframe
}
 
atom atexit()
{
	bind -delete repeater
}